function [FusionImage,EdgeMapPAN,Quality_Indices] = RGIndex(PAN_Image,MS_Image,nFusionMethod)
% RG Index Algorithm

% Published in: Arabian Journal of Geosciences, vol. 9, no. 1, p. 45, Jan. 2016.
% Title: "Quality assessment of pan-sharpening methods in high-resolution
% satellite images using radiometric and geometric index"
% Authors: M. Hasanlou and M. R. Saradjian

% Overall explanation. This code focuses on quality assessment of pansharpening of multispectral (MS)
% images with high-resolution panchromatic (Pan) images. Since most existing quality
% assessments take the entire image into account simultaneously and generate some uncertainties,
% a novel and rather objective quality index has been proposed for image fusion.
% The index is comprised of geometric and radiometric parts. Both geometric and radiometric measurements are calculated
% using morphological algorithm applied on an edge image to create a mask which is used to separate highfrequency regions
% from low-frequency ones.The accuracy assessment is made using common existing criteria on geometric and radiometric segments,
% and then a weighted sum is calculated to generate radiometric and geometric index (RG index).

% INPUTS:

% PAN_Image     -> Panchromatic image in one channel
% MS_Image      -> multispectral image (lines x rows x bands)
% nFusionMethod -> Index of pansharpening methods includes
%                  {'IHS','IHS*','PCA','GS','BT','EHL','HPM','SWT'} 

% OUTPUTS:

% FusionImage     -> Pansharpening image
% EdgeMapPAN      -> Extracred edge map from PAN image
% Quality_Indices -> Quality assessment by incorporating RG index using common 
%                    indices (SAM, SSIM, Correlation, UQI, RMSE, PSNR, SDD)


% Copyright (2017) School of Surveying and Geospatial Engineering, 
%                  College of Engineering, University of Tehran, Iran.

%%
ScaleFactor = size(PAN_Image,1)/size(MS_Image,1);

% Nearest-neighbor
MS_ImageResized = imresize(MS_Image,ScaleFactor,'nearest');

PAN_Image       = double(PAN_Image);
MS_ImageResized = double(MS_ImageResized);

%%
% setting parameters
R_Filter      = 0.09; % The Cutoff Frequency [0,0.5]
N_Order       = 1.0;  % The Order Of The Filter (n >= 1)

nRed   = 3;
nGreen = 2;
nBlue  = 1;

if nFusionMethod > 8
    error('nFusionMethod must be lower than 8');
    return;
end

switch nFusionMethod
    case 1 % IHS
        Intensity = round((MS_ImageResized(:,:,nRed)+MS_ImageResized(:,:,nGreen)+MS_ImageResized(:,:,nBlue))/3);
        Intensity = HistogramMatching(PAN_Image,Intensity,2^11);
        FusionImage(:,:,1) = MS_ImageResized(:,:,nRed  ) + PAN_Image - Intensity;
        FusionImage(:,:,2) = MS_ImageResized(:,:,nGreen) + PAN_Image - Intensity;
        FusionImage(:,:,3) = MS_ImageResized(:,:,nBlue ) + PAN_Image - Intensity;
    case 2 % IHS Modefied
        Intensity = round((MS_ImageResized(:,:,nRed)+MS_ImageResized(:,:,nGreen)+MS_ImageResized(:,:,nBlue))/3);
        t = 4;
        Intensity = HistogramMatching(PAN_Image,Intensity,2^11);
        FusionImage(:,:,1) = MS_ImageResized(:,:,nRed  ) - round(Intensity - PAN_Image + (PAN_Image - Intensity)/t);
        FusionImage(:,:,2) = MS_ImageResized(:,:,nGreen) - round(Intensity - PAN_Image + (PAN_Image - Intensity)/t);
        FusionImage(:,:,3) = MS_ImageResized(:,:,nBlue ) - round(Intensity - PAN_Image + (PAN_Image - Intensity)/t);
    case 3 % Principal Component Analysis
        [h,w,nBand] = size(MS_ImageResized);
        InputImage = Convert2d(MS_ImageResized);
        PAN_Image1   = Convert2d(PAN_Image);
        
        [pc,PCA_Image] = princomp(InputImage');
        MinV = min(min(PCA_Image(:,1)));
        MaxV = max(max(PCA_Image(:,1)));
        PCA_Image(:,1) = round((2^11-1) * (PCA_Image(:,1)-MinV)/(MaxV - MinV));
        %PAN_Image1 = HistogramMatching(PCA_Image(:,1),PAN_Image,2^11);
        PCA_Image(:,1) = PAN_Image1;
        FusionImage = pc * PCA_Image';
        FusionImage = Convert3d(FusionImage, h, w, nBand);
    case 4 % Gram-Schmidt
        
        PSF = fspecial('gaussian',3,1);
        blurredPAN = imfilter(PAN_Image,PSF,'replicate');
        
        MS_ImageResized(:,:,end+1) = blurredPAN;
        
        [h,w,nBand] = size(MS_ImageResized);
        InputImage = Convert2d(MS_ImageResized);
        PAN_Image1   = Convert2d(PAN_Image);
        MeuPAN     = mean(PAN_Image1);
        SigmaPAN   = std(PAN_Image1);
        
        [GS_Image, R] = grams(InputImage');
        %[GS_Image, R] = qr(InputImage',0);
        MeuGS1   = mean(GS_Image(:,1));
        SigmaGS1 = std(GS_Image(:,1));
        
        Gain = SigmaGS1/SigmaPAN;
        Bias = MeuGS1 - Gain * MeuPAN;
        
        ModeifedPAN = PAN_Image1 * Gain + Bias;
        
        ModeifedPAN = HistogramMatching(round(GS_Image(:,1)*100000)',round(100000*ModeifedPAN),2^11)/100000;
        GS_Image(:,1) = ModeifedPAN';
        FusionImage = R' * GS_Image';
        
        FusionImage = Convert3d(FusionImage, h, w, nBand);
    case 5 % Brovey Transform
        Intensity = (MS_ImageResized(:,:,nRed)+MS_ImageResized(:,:,nGreen)+MS_ImageResized(:,:,nBlue))/3;
        
        FusionImage(:,:,1) = MS_ImageResized(:,:,nRed  ) + round((PAN_Image - Intensity) .* (MS_ImageResized(:,:,nRed  ) ./ Intensity));
        FusionImage(:,:,2) = MS_ImageResized(:,:,nGreen) + round((PAN_Image - Intensity) .* (MS_ImageResized(:,:,nGreen) ./ Intensity));
        FusionImage(:,:,3) = MS_ImageResized(:,:,nBlue ) + round((PAN_Image - Intensity) .* (MS_ImageResized(:,:,nBlue ) ./ Intensity));

    case 6 % Ehler's Method
        R_Filter = 0.5;
        N_Order  = 1;
        Intensity = (MS_ImageResized(:,:,nRed)+MS_ImageResized(:,:,nGreen)+MS_ImageResized(:,:,nBlue))/3;
        Intensity_LPF = FFT_Filter(Intensity,'LPF',R_Filter,N_Order);
        PAN_Image_HPF  = FFT_Filter(PAN_Image ,'HPF',R_Filter,N_Order);
        
        IntensityNew = round(Intensity_LPF + PAN_Image_HPF);
        IntensityNew = HistogramMatching(IntensityNew,round(Intensity),2^12);
        
        FusionImage(:,:,1) = (MS_ImageResized(:,:,nRed  ) + PAN_Image - IntensityNew);
        FusionImage(:,:,2) = (MS_ImageResized(:,:,nGreen) + PAN_Image - IntensityNew);
        FusionImage(:,:,3) = (MS_ImageResized(:,:,nBlue ) + PAN_Image - IntensityNew);

    case 7 % High-Pass Modulation
        PAN_Image_LPF = FFT_Filter(PAN_Image,'LPF',R_Filter,N_Order);
        
        FusionImage(:,:,1) = MS_ImageResized(:,:,nRed  ) + ((PAN_Image - PAN_Image_LPF) .* (MS_ImageResized(:,:,nRed  ) ./ PAN_Image_LPF));
        FusionImage(:,:,2) = MS_ImageResized(:,:,nGreen) + ((PAN_Image - PAN_Image_LPF) .* (MS_ImageResized(:,:,nGreen) ./ PAN_Image_LPF));
        FusionImage(:,:,3) = MS_ImageResized(:,:,nBlue ) + ((PAN_Image - PAN_Image_LPF) .* (MS_ImageResized(:,:,nBlue ) ./ PAN_Image_LPF));
        
    case 8 % Schowengerdt
        Intensity = round((MS_ImageResized(:,:,nRed)+MS_ImageResized(:,:,nGreen)+MS_ImageResized(:,:,nBlue))/3);
        Intensity = HistogramMatching(PAN_Image,Intensity,2^11);
        PAN_Image_LPF = FFT_Filter(Intensity,'LPF',R_Filter,N_Order);
       
        FusionImage(:,:,1) = MS_ImageResized(:,:,nRed  ) + ((PAN_Image - PAN_Image_LPF));
        FusionImage(:,:,2) = MS_ImageResized(:,:,nGreen) + ((PAN_Image - PAN_Image_LPF));
        FusionImage(:,:,3) = MS_ImageResized(:,:,nBlue ) + ((PAN_Image - PAN_Image_LPF));
end

[FusionImageGrayEdge,EdgeMapPAN,EdgeMapfill] = DetectEdge(PAN_Image,2^11-1,1);

Quality_Indices = MyQuality_Fusion(MS_ImageResized,[nRed,nGreen,nBlue],PAN_Image,EdgeMapPAN,FusionImage);

%------
function OutputImage = HistogramMatching(InputImage1,InputImage2,nBin,bShow)

if nargin == 3
    bShow = 0;
end

[N,M] = size(InputImage1);
HistogramInput1  = myhist(InputImage1,nBin);
% CumulativeInput1 = cumsum(HistogramInput1);

% Normalize hgram
HistogramInput1 = HistogramInput1*(numel(InputImage2)/sum(HistogramInput1));       % Set sum = numel(a)
m = length(HistogramInput1);

HistogramInput2  = myhist(InputImage2,nBin);
CumulativeInput2 = cumsum(HistogramInput2);


T = createTransformationToIntensityImage(InputImage2,HistogramInput1,m,nBin,HistogramInput2',CumulativeInput2');

if 0
    % OutputImage = grayxform(uint16(InputImage2), T);
    for i=1:size(InputImage2,1)
        for j=1:size(InputImage2,2)
            %OutputImage(i,j) = round((nBin-1) * T(InputImage2(i,j)) /(N * M));
            OutputImage(i,j) = round((nBin-1) * T(InputImage2(i,j)));
        end
    end
else
     OutputImage = round((nBin-1) * T(InputImage2(:)+1));
     OutputImage = reshape(OutputImage,size(InputImage2,1),size(InputImage2,2));
end

if bShow
    HistogramOut  = myhist(OutputImage,nBin);
    figure
    colormap gray
    subplot 321, plot(HistogramInput1)
    subplot 322, imagesc(InputImage1),axis image,axis off
    subplot 323, plot(HistogramInput2)
    subplot 324, imagesc(InputImage2),axis image,axis off
    subplot 325, plot(HistogramOut);
    subplot 326, imagesc(OutputImage),axis image,axis off
    colormap gray
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function T = createTransformationToIntensityImage(a,hgram,m,n,nn,cum)

cumd = cumsum(hgram*numel(a)/sum(hgram));

% Create transformation to an intensity image by minimizing the error
% between desired and actual cumulative histogram.
tol = ones(m,1)*min([nn(1:n-1),0;0,nn(2:n)])/2;
err = (cumd(:)*ones(1,n)-ones(m,1)*cum(:)')+tol;
d = find(err < -numel(a)*sqrt(eps));
if ~isempty(d)
   err(d) = numel(a)*ones(size(d));
end
[dum,T] = min(err); %#ok
T = (T-1)/(m-1);

%------
function histogram = myhist(InputImage,nBin)

histogram = zeros(nBin,1);
for i=1:size(InputImage,1)
    for j=1:size(InputImage,2)
        histogram(InputImage(i,j)+1,1) = histogram(InputImage(i,j)+1,1) +1;
    end
end

% ----
function Filter_Image = FFT_Filter(InputImage,strFilter,R,N_Order)

Y = imfft(InputImage);
if strFilter == 'LPF'
    filter = lowpassfilter(size(Y), R, N_Order);
else 
    filter = highpassfilter(size(Y), R, N_Order);
end
Y = filter .* Y;
Filter_Image = imifft(Y);
return;
[n,m]=size(InputImage);
r = ones(n,m);
r(1:2:end,2:2:end) = -1;
r(2:2:end,1:2:end) = -1;
img  = double(InputImage);
rimg = r .* img;

if strFilter == 'LPF'
    filter = zeros(n,m);
    value  = 1;
else 
    filter = ones(n,m);
    value  = 0;
end

if mod(n,2)
    n=n+1;
end
if mod(m,2)
    m=m+1;
end
for x=-R:R
    y = round(sqrt(R^2-x^2));
    filter(n/2+x,m/2-y:m/2+y) = value;
end

fimg  = fft2(rimg);
flimg = fimg.*filter;
iimg  = ifft2(flimg);
Filter_Image = round(abs(iimg));

% ----
function Y = imfft(X)
Y = fftshift(fft2(X));

% ----
function Y = imifft(X)
Y = abs(ifft2(ifftshift(X)));

% ----
function f = highpassfilter(sze, cutoff, n)
% HIGHPASSFILTER  - Constructs a high-pass butterworth filter.
%
% usage: f = highpassfilter(sze, cutoff, n)
%
% where: sze    is a two element vector specifying the size of filter
%               to construct.
%        cutoff is the cutoff frequency of the filter 0 - 0.5
%        n      is the order of the filter, the higher n is the sharper
%               the transition is. (n must be an integer >= 1).
%
% The frequency origin of the returned filter is at the corners.
%
% See also: LOWPASSFILTER, HIGHBOOSTFILTER, BANDPASSFILTER
%

% Peter Kovesi   pk@cs.uwa.edu.au
% Department of Computer Science & Software Engineering
% The University of Western Australia
%
% October 1999

if cutoff < 0 | cutoff > 0.5
    error('cutoff frequency must be between 0 and 0.5');
end

if rem(n,1) ~= 0 | n < 1
    error('n must be an integer >= 1');
end

f = 1.0 - lowpassfilter(sze, cutoff, n);

% ----
function f = lowpassfilter(sze, cutoff, n)
% LOWPASSFILTER - Constructs a low-pass butterworth filter.
%
% usage: f = lowpassfilter(sze, cutoff, n)
%
% where: sze    is a two element vector specifying the size of filter
%               to construct.
%        cutoff is the cutoff frequency of the filter 0 - 0.5
%        n      is the order of the filter, the higher n is the sharper
%               the transition is. (n must be an integer >= 1).
%
% The frequency origin of the returned filter is at the corners.
%
% See also: HIGHPASSFILTER, HIGHBOOSTFILTER, BANDPASSFILTER
%

% Peter Kovesi   pk@cs.uwa.edu.au
% Department of Computer Science & Software Engineering
% The University of Western Australia
%
% October 1999
%
% Modified Rob Gaddi   gaddi@rice.edu
% ELEC 301
% Rice University
%
% December 2001

if cutoff < 0 | cutoff > 0.5
    error('cutoff frequency must be between 0 and 0.5');
end

if rem(n,1) ~= 0 | n < 1
    error('n must be an integer >= 1');
end

%  Modification ELEC 301 Project Group, Dec 2001
%  Original code [rows, cols] = sze was not accepted by Matlab
rows = sze(1);
cols = sze(2);
%  End Alteration

% X and Y matrices with ranges normalised to +/- 0.5
x =  (ones(rows,1) * [1:cols]  - (fix(cols/2)+1))/cols;
y =  ([1:rows]' * ones(1,cols) - (fix(rows/2)+1))/rows;

radius = sqrt(x.^2 + y.^2);        % A matrix with every pixel = radius relative to centre.

%  Alteration, ELEC 301 Project Group, Dec 2001
%  Original code fftshifted the filter before output.  Since
%  imFFT and imIFFT already shift, the output should remain low-centered.
f = 1 ./ (1.0 + (radius ./ cutoff).^(2*n));   % The filter
%  End Alteration

% ----
function [SegmentedEdges,BWdilate,BWfill] = DetectEdge(I,bin,nMethods)

switch nMethods
    case 1 % Canny
        BW = edge(I, 'canny',0.02);
    case 2 % Prewitt
        BW = edge(I, 'canny',0.01);
%         BW = edge(I, 'prewitt');
    case 3 % Zero-Cross
        BW = edge(I, 'canny',0.02);
        %BW = edge(I, 'zerocross');
    case 4 % Laplacian of Gaussian
        BW = edge(I, 'canny',0.03);
        %BW = edge(I, 'log');
    case 5 % Roberts
        BW = edge(I, 'canny',0.04);
        %BW = edge(I, 'roberts');
    case 6 % Sobel
        BW = edge(I, 'canny',0.05);
        %BW = edge(I, 'sobel');        
end

se90 = strel('line', 2, 90);
se0 = strel('line', 2, 0);

BWdilate = imdilate(BW, [se90 se0]);

SegmentedEdges = I;
SegmentedEdges(BWdilate) = bin;
BWfill = BW;%dilate;


%------
function Quality_Indices = MyQuality_Fusion(MSImage3D,IndexMS,PANImage2D,EdgeMapPAN,FusionImage3D)

FusionImage3D = double(FusionImage3D);
if size(FusionImage3D,3) > 3
    FusionImage3D = FusionImage3D(:,:,1:3);
end

for i=1:size(IndexMS,2)
    MSImage3D_selected(:,:,i) = MSImage3D(:,:,IndexMS(i));
end
MSImageGray = rgb2gray(uint16(MSImage3D_selected));

Quality_Indices =[];
FusionImageGray = rgb2gray(uint16(FusionImage3D));

PANImageGrayEdgeMap = Convert2d(EdgeMapPAN);
FusionImage         = Convert2d(FusionImageGray);
PANImage            = Convert2d(PANImage2D);
MSImage             = Convert2d(MSImageGray);

FusionImage_Geometric   = FusionImage(find(PANImageGrayEdgeMap));
FusionImage_Radiometric = FusionImage(find(~PANImageGrayEdgeMap));

PANImage_Geometric   = PANImage(find(PANImageGrayEdgeMap));
MSImage_Radiometric  = MSImage(find(~PANImageGrayEdgeMap));

Wr = size(FusionImage_Radiometric,2)/size(FusionImage,2);
Wg = size(FusionImage_Geometric,2)/size(FusionImage,2);

% Spectral Angle Mapper
Quality_Indices.SAM(1) = Sam(FusionImage_Geometric'  , PANImage_Geometric')*180/pi;
Quality_Indices.SAM(2) = Sam(FusionImage_Radiometric', MSImage_Radiometric')*180/pi;
Quality_Indices.SAM(4) = Sam(FusionImage', MSImage')*180/pi;

Sg = Quality_Indices.SAM(1);
Sr = Quality_Indices.SAM(2);
Quality_Indices.SAM(3) = (Wr*Sr + Wg*Sg);

% Rroot Mean Square Error (RMSE)
Quality_Indices.RMSE(1) = sqrt(sum((FusionImage_Geometric'  - PANImage_Geometric' ).^2)/size(PANImage_Geometric ,2)-1);
Quality_Indices.RMSE(2) = sqrt(sum((FusionImage_Radiometric'- MSImage_Radiometric').^2)/size(MSImage_Radiometric,2)-1);
Quality_Indices.RMSE(4) = sqrt(sum((FusionImage'- MSImage').^2)/size(MSImage,2)-1);

Sg = Quality_Indices.RMSE(1);
Sr = Quality_Indices.RMSE(2);
% Quality_Indices.RMSE(3) = (Wr*Sr + Wg*Sg);
Quality_Indices.RMSE(3) = sqrt(Wr^2*Sr^2 + Wg^2*Sg^2);

% Correlation coefficients
Corr = corrcoef(FusionImage_Geometric,PANImage_Geometric);
Quality_Indices.Correlation(1) = Corr(1,2);
Corr = corrcoef(FusionImage_Radiometric, MSImage_Radiometric);
Quality_Indices.Correlation(2) = Corr(1,2);
Corr = corrcoef(FusionImage, MSImage);
Quality_Indices.Correlation(4) = Corr(1,2);

Sg = Quality_Indices.Correlation(1);
Sr = Quality_Indices.Correlation(2);
Quality_Indices.Correlation(3) = (Wr*Sr + Wg*Sg);

% Q index
Quality_Indices.UQI(1) = UQI(FusionImage_Geometric,PANImage_Geometric)*10;
Quality_Indices.UQI(2) = UQI(FusionImage_Radiometric,MSImage_Radiometric)*10;
Quality_Indices.UQI(4) = UQI(FusionImage,MSImage)*10;

Sg = Quality_Indices.UQI(1);
Sr = Quality_Indices.UQI(2);
Quality_Indices.UQI(3) = (Wr*Sr + Wg*Sg);

% SDD
Quality_Indices.SDD(1) = std(FusionImage_Geometric - PANImage_Geometric);
Quality_Indices.SDD(2) = std(FusionImage_Radiometric - MSImage_Radiometric);
Quality_Indices.SDD(4) = std(FusionImage - MSImage);

Sg = Quality_Indices.SDD(1);
Sr = Quality_Indices.SDD(2);
Quality_Indices.SDD(3) = (Wr*Sr + Wg*Sg);

% Structural SIMilarity (SSIM)
Quality_Indices.SSIM(1) = ssim(FusionImage_Geometric,PANImage_Geometric);
Quality_Indices.SSIM(2) = ssim(FusionImage_Radiometric,MSImage_Radiometric);
Quality_Indices.SSIM(4) = ssim(FusionImage,MSImage);

Sg = Quality_Indices.SSIM(1);
Sr = Quality_Indices.SSIM(2);
Quality_Indices.SSIM(3) = (Wr*Sr + Wg*Sg);

% Peak Signal-to-Noise Ratio (PSNR)
Quality_Indices.PSNR(1) = psnr(FusionImage_Geometric,PANImage_Geometric);
Quality_Indices.PSNR(2) = psnr(FusionImage_Radiometric,MSImage_Radiometric);
Quality_Indices.PSNR(4) = psnr(FusionImage,MSImage);

Sg = Quality_Indices.PSNR(1);
Sr = Quality_Indices.PSNR(2);
Quality_Indices.PSNR(3) = (Wr*Sr + Wg*Sg);


% 
% Quality_Indices =[Quality_Indices.SAM;Quality_Indices.SSIM;...
%         Quality_Indices.Correlation;Quality_Indices.UQI;...
%         Quality_Indices.RMSE;Quality_Indices.PSNR;Quality_Indices.SDD];

% ----
function [img] = Convert3d(img, h, w, numBands)

if (ndims(img) ~= 2)
    error('Input image must be p x N.');
end

[numBands, N] = size(img);

img = reshape(img.', h, w, numBands); 

% ----
function [M] = Convert2d(M)

if(ndims(M)==3)
    
    if (ndims(M) ~= 3)
        error('Input image must be m x n x p.');
    end
    
    [h, w, numBands] = size(M);
    M = double(reshape(M, w*h, numBands).');
else
    [h, w] = size(M);
    
    M = double(reshape(M, w*h, 1).');
end

%------
function [errRadians] = Sam(a, b)
% SAM Computes the spectral angle error (in radians) between two vectors
%
% Usage
%   [errRadians] = Sam(a, b)
% Inputs
%   a - Vector 1.
%   b - Vector 2.
% Outputs
%   errRadians - angle between vectors a and b in radians

[p,N] = size(a);
errRadians = zeros(N,1);
for k=1:N
    tmp = a(:,k);
    errRadians(k) = acos(dot(tmp, b)/ (norm(b) * norm(tmp)));
end

%------
function Quality_Indices = UQI(Im1,Im2)

Std_MS = std(Im1);
Std_Fu = std(Im2);
Mu_MS  = mean(Im1);
Mu_Fu  = mean(Im2);
Co_Fu_MS = cov(Im1,Im2);
Co_Fu_MS = sqrt(Co_Fu_MS(1,2));
Quality_Indices = 4 * Co_Fu_MS * Mu_MS * Mu_Fu/((Mu_MS^2 + Mu_Fu^2)*(Std_MS^2 + Std_Fu^2));

%------
function [Q, R] = grams(A)

% grams  Gram-Schmidt orthogonalization of the columns of A.
% The columns of A are assumed to be linearly independent.
%
% Q = grams(A) returns an m by n matrix Q whose columns are 
% an orthonormal basis for the column space of A.
%
% [Q, R] = grams(A) returns a matrix Q with orthonormal columns
% and an invertible upper triangular matrix R so that A = Q*R.
%
% Warning: For a more stable algorithm, use [Q, R] = qr(A, 0) .

[m, n] = size(A);
Asave = A;
for j = 1:n
  for k = 1:j-1
    mult = (A(:, j)'*A(:, k)) / (A(:, k)'*A(:, k));
    A(:, j) = A(:, j) - mult*A(:, k);
  end
end
for j = 1:n
  if norm(A(:, j)) < sqrt(eps)
    error('Columns of A are linearly dependent.')
  end
  Q(:, j) = A(:, j) / norm(A(:, j));
end
R = Q'*Asave; 